"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var govern_1 = require("govern");
var PriorityContext = React.createContext(1);
function createSubscribe(to, render) {
    return Subscribe.Element({ to: to, render: render });
}
exports.createSubscribe = createSubscribe;
/**
 * Accepts an observable as a prop, and passes each of its values to the
 * component's children via a render function.
 *
 * NOTE: this component should be generic on output type, but TSX doesn't
 * properly support generic components. In the meantime, if you want
 * typing, use the `createSubscribe` factory.
 *
 * See https://github.com/Microsoft/TypeScript/issues/14729.
 */
var Subscribe = /** @class */ (function (_super) {
    __extends(Subscribe, _super);
    function Subscribe() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.renderWithPriority = function (priority) {
            return React.createElement(InnerSubscribe, __assign({}, _this.props, { priority: priority }));
        };
        return _this;
    }
    /**
     * A factory to create a `<Subscribe to>` React element, with typing.
     * @param store The store to subscribe to
     * @param children A function to render each of the store's values.
     */
    Subscribe.Element = function (props) {
        return React.createElement(Subscribe, props);
    };
    Subscribe.prototype.render = function () {
        return React.createElement(PriorityContext.Consumer, { children: this.renderWithPriority });
    };
    return Subscribe;
}(React.Component));
exports.Subscribe = Subscribe;
var InnerSubscribe = /** @class */ (function (_super) {
    __extends(InnerSubscribe, _super);
    function InnerSubscribe(props) {
        var _this = _super.call(this, props) || this;
        _this.receiveError = function (error) {
            // Grab errors from the store, and throw them so a higher level React
            // component can handle it with componentDidCatch.
            throw error;
        };
        _this.handleChange = function (snapshot, dispatch) {
            _this.isAwaitingRenderFromFlush = true;
            _this.isAwaitingRenderFromProps = false;
            _this.setState({
                snapshot: snapshot.snapshot,
                dispatch: dispatch
            });
        };
        _this.handleStartDispatch = function () {
            _this.isDispatching = true;
        };
        _this.handleEndDispatch = function () {
            _this.isDispatching = false;
            if (_this.isAwaitingRenderFromProps) {
                // Now that `isDispatching` is false, `shouldComponentUpdate` will return
                // true, so we can flush through our changes by setting a dummy object.
                _this.setState({ dummy: {} });
            }
        };
        _this.state = {};
        _this.isDispatching = false;
        if (!_this.props.to) {
            console.warn("A \"to\" prop must be provided to <Subscribe> but \"" + _this.props.to + "\" was received.");
        }
        _this.observable = govern_1.createObservable(Binding.Element({ initialElement: _this.props.to }));
        _this.isAwaitingRenderFromFlush = true;
        _this.isAwaitingRenderFromProps = false;
        _this.state = {
            snapshot: _this.observable.getValue().snapshot,
            dispatch: _this.observable.waitUntilNotFlushing,
        };
        // Create subsription within `componentWillMount` instead of in
        // `constructor`, as it is possible for components to have side effects
        // on initial subscription (as the first `componentDidFlush` is held
        // until then, and it can cause `setState`.
        _this.subscription = _this.observable.subscribe(_this.handleChange, _this.receiveError, undefined, _this.handleStartDispatch, _this.handleEndDispatch, String(_this.props.priority));
        return _this;
    }
    InnerSubscribe.prototype.UNSAFE_componentWillReceiveProps = function (nextProps) {
        if (!nextProps.to) {
            console.warn("A \"to\" prop must be provided to <Subscribe> but \"" + this.props.to + "\" was received.");
        }
        // If no flush is received during dispatch, we'll want to re-render
        // manually when it completes.
        if (this.isDispatching) {
            this.isAwaitingRenderFromProps = true;
        }
        // As elements are immutable, we can skip a lot of updates by
        // checking if the `to` element/store has changed.
        if (nextProps.to !== this.props.to) {
            this.observable.getValue().changeElement(nextProps.to);
        }
    };
    InnerSubscribe.prototype.componentDidCatch = function (error) {
        this.cleanup();
    };
    InnerSubscribe.prototype.componentWillUnmount = function () {
        this.cleanup();
    };
    InnerSubscribe.prototype.cleanup = function () {
        if (this.observable) {
            this.subscription.unsubscribe();
            this.observable.dispose();
            delete this.subscription;
            delete this.observable;
        }
    };
    InnerSubscribe.prototype.shouldComponentUpdate = function () {
        return !this.isDispatching || this.isAwaitingRenderFromFlush;
    };
    InnerSubscribe.prototype.render = function () {
        this.isAwaitingRenderFromFlush = false;
        this.isAwaitingRenderFromProps = false;
        var children = this.props.children || this.props.render;
        return children(this.state.snapshot, this.state.dispatch);
    };
    return InnerSubscribe;
}(React.Component));
exports.InnerSubscribe = InnerSubscribe;
var Binding = /** @class */ (function (_super) {
    __extends(Binding, _super);
    function Binding(props) {
        var _this = _super.call(this, props) || this;
        _this.changeElement = function (element) {
            _this.setState({
                element: element,
            });
        };
        _this.state = {
            element: _this.props.initialElement
        };
        return _this;
    }
    Binding.Element = function (props) {
        return govern_1.createElement(Binding, props);
    };
    Binding.prototype.render = function () {
        return govern_1.combine({
            snapshot: this.state.element,
            changeElement: this.changeElement,
        });
    };
    Binding.prototype.shouldComponentPublish = function (prevProps, prevState, prevSubs) {
        return prevSubs.snapshot !== this.subs.snapshot;
    };
    return Binding;
}(govern_1.Component));
//# sourceMappingURL=Subscribe.js.map