(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('govern')) :
	typeof define === 'function' && define.amd ? define(['exports', 'react', 'govern'], factory) :
	(factory((global.ReactGovern = {}),global.React,global.Govern));
}(this, (function (exports,React,govern) { 'use strict';

var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
function createStore(element, render) {
    return Store.Element({ element: element, render: render });
}
/**
 * Accepts an observable as a prop, and passes each of its values to the
 * component's children via a render function.
 *
 * NOTE: this component should be generic on output type, but TSX doesn't
 * properly support generic components. In the meantime, if you want
 * typing, use the `createSubscribe` factory.
 *
 * See https://github.com/Microsoft/TypeScript/issues/14729.
 */
var Store = /** @class */ (function (_super) {
    __extends(Store, _super);
    function Store(props) {
        var _this = _super.call(this, props) || this;
        _this.observable = govern.createObservable(govern.createElement(Binding, { initialElement: _this.props.element }));
        return _this;
    }
    Store.Element = function (props) {
        return React.createElement(Store, props);
    };
    Store.prototype.componentWillMount = function () {
        if (!this.props.element) {
            console.warn("An \"element\" prop must be provided to <Observable> but \"" + this.props.element + "\" was received.");
        }
    };
    Store.prototype.UNSAFE_componentWillReceiveProps = function (nextProps) {
        if (!nextProps.element) {
            console.warn("A \"element\" prop must be provided to <Observable> but \"" + this.props.element + "\" was received.");
        }
        // As elements are immutable, we can skip a lot of updates by
        // checking if the `to` element/store has changed.
        if (nextProps.element !== this.props.element) {
            this.observable.getValue().changeElement(nextProps.element);
        }
    };
    Store.prototype.componentDidCatch = function (error) {
        this.disposeStore();
    };
    Store.prototype.componentWillUnmount = function () {
        this.disposeStore();
    };
    Store.prototype.disposeStore = function () {
        if (this.observable) {
            this.observable.dispose();
            delete this.observable;
        }
    };
    Store.prototype.render = function () {
        var render = (this.props.children || this.props.render);
        return render(this.observable);
    };
    return Store;
}(React.Component));
var Binding = /** @class */ (function (_super) {
    __extends(Binding, _super);
    function Binding(props) {
        var _this = _super.call(this, props) || this;
        _this.changeElement = function (element) {
            _this.setState({
                element: element,
            });
        };
        _this.state = {
            element: _this.props.initialElement
        };
        return _this;
    }
    Binding.prototype.render = function () {
        return govern.combine({
            snapshot: this.state.element,
            changeElement: this.changeElement,
        });
    };
    Binding.prototype.getPublishedValue = function () {
        this.subs;
        return this.subs;
    };
    Binding.prototype.shouldComponentPublish = function (prevProps, prevState, prevSubs) {
        return prevSubs.snapshot !== this.subs.snapshot;
    };
    return Binding;
}(govern.Component));

var __extends$1 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var PriorityContext = React.createContext(1);
function createSubscribe(to, render) {
    return Subscribe.Element({ to: to, render: render });
}
/**
 * Accepts an observable as a prop, and passes each of its values to the
 * component's children via a render function.
 *
 * NOTE: this component should be generic on output type, but TSX doesn't
 * properly support generic components. In the meantime, if you want
 * typing, use the `createSubscribe` factory.
 *
 * See https://github.com/Microsoft/TypeScript/issues/14729.
 */
var Subscribe = /** @class */ (function (_super) {
    __extends$1(Subscribe, _super);
    function Subscribe() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.renderWithPriority = function (priority) {
            return React.createElement(InnerSubscribe, __assign({}, _this.props, { priority: priority }));
        };
        return _this;
    }
    /**
     * A factory to create a `<Subscribe to>` React element, with typing.
     * @param store The store to subscribe to
     * @param children A function to render each of the store's values.
     */
    Subscribe.Element = function (props) {
        return React.createElement(Subscribe, props);
    };
    Subscribe.prototype.render = function () {
        return React.createElement(PriorityContext.Consumer, { children: this.renderWithPriority });
    };
    return Subscribe;
}(React.Component));
var InnerSubscribe = /** @class */ (function (_super) {
    __extends$1(InnerSubscribe, _super);
    function InnerSubscribe(props) {
        var _this = _super.call(this, props) || this;
        _this.receiveError = function (error) {
            // Grab errors from the store, and throw them so a higher level React
            // component can handle it with componentDidCatch.
            throw error;
        };
        _this.handleChange = function (snapshot, dispatch) {
            _this.isAwaitingRenderFromFlush = true;
            _this.isAwaitingRenderFromProps = false;
            _this.setState({
                snapshot: snapshot.snapshot,
                dispatch: dispatch
            });
        };
        _this.handleStartDispatch = function () {
            _this.isDispatching = true;
        };
        _this.handleEndDispatch = function () {
            _this.isDispatching = false;
            if (_this.isAwaitingRenderFromProps) {
                // Now that `isDispatching` is false, `shouldComponentUpdate` will return
                // true, so we can flush through our changes by setting a dummy object.
                _this.setState({ dummy: {} });
            }
        };
        _this.state = {};
        _this.isDispatching = false;
        if (!_this.props.to) {
            console.warn("A \"to\" prop must be provided to <Subscribe> but \"" + _this.props.to + "\" was received.");
        }
        _this.observable = govern.createObservable(Binding$1.Element({ initialElement: _this.props.to }));
        _this.isAwaitingRenderFromFlush = true;
        _this.isAwaitingRenderFromProps = false;
        _this.state = {
            snapshot: _this.observable.getValue().snapshot,
            dispatch: _this.observable.waitUntilNotFlushing,
        };
        // Create subsription within `componentWillMount` instead of in
        // `constructor`, as it is possible for components to have side effects
        // on initial subscription (as the first `componentDidFlush` is held
        // until then, and it can cause `setState`.
        _this.subscription = _this.observable.subscribe(_this.handleChange, _this.receiveError, undefined, _this.handleStartDispatch, _this.handleEndDispatch, String(_this.props.priority));
        return _this;
    }
    InnerSubscribe.prototype.UNSAFE_componentWillReceiveProps = function (nextProps) {
        if (!nextProps.to) {
            console.warn("A \"to\" prop must be provided to <Subscribe> but \"" + this.props.to + "\" was received.");
        }
        // If no flush is received during dispatch, we'll want to re-render
        // manually when it completes.
        if (this.isDispatching) {
            this.isAwaitingRenderFromProps = true;
        }
        // As elements are immutable, we can skip a lot of updates by
        // checking if the `to` element/store has changed.
        if (nextProps.to !== this.props.to) {
            this.observable.getValue().changeElement(nextProps.to);
        }
    };
    InnerSubscribe.prototype.componentDidCatch = function (error) {
        this.cleanup();
    };
    InnerSubscribe.prototype.componentWillUnmount = function () {
        this.cleanup();
    };
    InnerSubscribe.prototype.cleanup = function () {
        if (this.observable) {
            this.subscription.unsubscribe();
            this.observable.dispose();
            delete this.subscription;
            delete this.observable;
        }
    };
    InnerSubscribe.prototype.shouldComponentUpdate = function () {
        return !this.isDispatching || this.isAwaitingRenderFromFlush;
    };
    InnerSubscribe.prototype.render = function () {
        this.isAwaitingRenderFromFlush = false;
        this.isAwaitingRenderFromProps = false;
        var children = this.props.children || this.props.render;
        return children(this.state.snapshot, this.state.dispatch);
    };
    return InnerSubscribe;
}(React.Component));
var Binding$1 = /** @class */ (function (_super) {
    __extends$1(Binding, _super);
    function Binding(props) {
        var _this = _super.call(this, props) || this;
        _this.changeElement = function (element) {
            _this.setState({
                element: element,
            });
        };
        _this.state = {
            element: _this.props.initialElement
        };
        return _this;
    }
    Binding.Element = function (props) {
        return govern.createElement(Binding, props);
    };
    Binding.prototype.render = function () {
        return govern.combine({
            snapshot: this.state.element,
            changeElement: this.changeElement,
        });
    };
    Binding.prototype.shouldComponentPublish = function (prevProps, prevState, prevSubs) {
        return prevSubs.snapshot !== this.subs.snapshot;
    };
    return Binding;
}(govern.Component));

var __extends$2 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$1 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
/**
 * A Higher Order Component version of the <Store of /> component.
 */
var withStore = function (mapOwnPropsToElement, propName) {
    if (propName === void 0) { propName = 'store'; }
    var mapFn = mapOwnPropsToElement;
    if (govern.isValidElement(mapOwnPropsToElement)) {
        mapFn = (function () { return mapOwnPropsToElement; });
    }
    else if (mapOwnPropsToElement instanceof govern.Component.constructor) {
        mapFn = function (props) { return govern.createElement(mapOwnPropsToElement, props); };
    }
    if (typeof mapFn !== 'function') {
        throw new Error("\"govern(..., mapFn)\" received an unknown \"mapFn\" of type \"" + typeof mapFn + "\" for \"" + String(mapOwnPropsToElement) + "\".");
    }
    var decorator = function (WrappedComponent) {
        // Use a class component instead of a stateless functional component so
        // that consumers can use refs if they need.
        return /** @class */ (function (_super) {
            __extends$2(SubscribeWrapper, _super);
            function SubscribeWrapper() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            SubscribeWrapper.prototype.render = function () {
                var _this = this;
                return Store.Element({
                    element: mapFn(this.props),
                    children: function (store) {
                        var _a;
                        return React.createElement(WrappedComponent, __assign$1({}, _this.props, (_a = {}, _a[propName] = store, _a)));
                    }
                });
            };
            return SubscribeWrapper;
        }(React.Component));
    };
    return decorator;
};

exports.createStore = createStore;
exports.Store = Store;
exports.createSubscribe = createSubscribe;
exports.Subscribe = Subscribe;
exports.InnerSubscribe = InnerSubscribe;
exports.withStore = withStore;

Object.defineProperty(exports, '__esModule', { value: true });

})));
